'***********************************************************************
'
'    Filename        :     Seifenblasen-Maschine
'    File Version    :     1.0
'    Datum           :     13.03.2009            	
'    Autor           :     Peter Verderber		
'    Funktion        :     Seifenblasen-Maschine		
'    Letzte Revision :     1.0
'    Datum           :     13.03.2009           
'    Ziel PIC        :     16F872
'    Compiler        :     iL_BAS16 - Professionell  V5.7-10  23.11.2007 
'    
'***********************************************************************
'*****                Definitionen und Konstanten                  *****
'***********************************************************************

Date
Time

$NCALDEF

$LIST BIN          'erzeugt zustzlich eine Binrdatei 
'$LIST OBJ2HEX     'das Hexfile bekommt anstatt OBJ die Dateiendung HEX   

define device 16F872, xt_osc, wdt_off, psa1, psv7, adcfg6
                       
xtal 4.0 

tris ra,%00111100
tris rb,%00000000          
tris rc,%11111111    

output rb,%00000000
 
define zyklus         = auto as word         'Anzahl der Zyklen seit Start
define zyklen         = auto as word         'Summe aller Zyklen
define zyklenL        = zyklen as byte       'Summe aller Zyklen L-Byte
define zyklenH        = zyklen + 1 as byte   'Summe aller Zyklen H-Byte

define zykl_wd        = auto as word         'Wartungsintervall
define zykl_wdL       = zykl_wd as byte      'Wartungsintervall  L-Byte
define zykl_wdH       = zykl_wd + 1 as byte  'Wartungsintervall  H-Byte

define hhe           = auto as word   'aktuelle Fllhhe (max. 1900 Imp.)
define inhalt         = auto as word   'Tankinhalt in Impulse
define volumen        = auto as byte   'Tankinhalt in Milli-Liter
define menu           = auto as byte   'Auswahl Mens 
define text           = auto as byte   'Auswahl Texte
 
define sensor_flag    = auto as byte   'Tank: 0 = leer, 1 = voll 
define init_flag      = auto as byte   'Initialisierung 0 = not ok, 1 = ok 
define stop_flag      = auto as byte   'Stop-Taste: 0 = nicht gedrckt, 
                                       '            1 = gedrckt

define summen_symb    = auto as byte   'Summen-Symbol

define stop_led       = ra,0           'Ausgang Stop-LED an / aus                                             
define hupe           = ra,1           'Ausgang Hupe an / aus                          
define taste_info     = ra,4           'Eingang Info-Taste
 
define motor          = rb,6           'Ausgang Antriebsmotor EIN / AUS
define ventilator     = rb,7           'Ausgang Ventilator    EIN / AUS 

define tankdeckel     = rc,0           'Eingang Tankdeckel           
define endschalter    = rc.1           'Eingang Endschalter 
define sensor         = rc,2           'Eingang Feuchtesensor
define in1            = rc,3           'Eingang Impulsreihe 1   
define in2            = rc,4           'Eingang Impulsreihe 2 
define taste_start    = rc,5           'Eingang Start-Taste  
define taste_stop     = rc,6           'Eingang Stop-Taste
define taste_notaus   = rc,7           'Eingang Not-Aus-Taste   

goto main                              'Start Hauptprogramm

'*************************************************************************
'*****                    Initialisierungen                          *****
'*************************************************************************

init:
  res stop_led                         'Port RA initalisieren
  res hupe
                                  
  output rb,%00000000                  'Port RB initalisieren
  
  wait 1000                            'LCD initalisieren
  lcdinit rb,2,16
  lcdclear    '"1234567890123456"         
  lcdwrite 1,1,"  Seifenblasen  "
  lcdwrite 2,1,"    Maschine    "
  wait 2000
  lcdwrite 1,1,"  designed  by  "
  lcdwrite 2,1,"PVC - electronic"
  wait 2000
  lcdclear 
    
return

'------------------------------------------------------------------------- 
  
init_var:                              'Variablen initalisieren
  let init_flag   = 0
  let sensor_flag = 0
  let stop_flag   = 0
  let hhe        = 0
  let inhalt      = 0 
  let menu        = 0
  let text        = 0
  let volumen     = 0
  let zyklus      = 0
  let summen_symb = 246
return       

'***********************************************************************
'*****         Daten im EEPROM speichern bzw. auslesen             *****
'*********************************************************************** 

store_zyklen:              
  write $00, zyklenL                     'Gesamtanzahl Zyklen speichern
  wait 10
  write $01, zyklenH 
  wait 10
  write $02, zykl_wdL          'Anzahl Zyklen bis zur Wartung speichern
  wait 10
  write $03, zykl_wdH 
  wait 10
return

'------------------------------------------------------------------------- 

read_zyklen:               
  read $00, zyklenL                      'Gesamtanzahl Zyklen auslesen
  wait 10
  read $01, zyklenH 
  wait 10
  read $02, zykl_wdL           'Anzahl Zyklen bis zur Wartung auslesen
  wait 10
  read $03, zykl_wdH 
  wait 10
return

'*************************************************************************
'*****                     Unterprogramme                            *****
'*************************************************************************

watch_dog:                                       'Watchdog
  if tankdeckel   = 0 then gosub tank_deckel
  if taste_notaus = 0 then gosub not_aus
  if zykl_wd > 499 then goto wartung
return 
  
'-------------------------------------------------------------------------
  
not_aus:                               'Strmeldung: NOT-AUS-Schalter
  let text = 0
  wait 10 
  if taste_notaus = 0 then goto error_disp
return
  
'-------------------------------------------------------------------------  

tank_leer:                             'Strmeldung: Tank leer
  let text = 1
goto error_disp
      
'-------------------------------------------------------------------------  

tank_deckel:                           'Strmeldung: Tankdeckel fehlt 
  let text = 2
  wait 10
  if tankdeckel = 0 then goto error_disp
return

'-------------------------------------------------------------------------  

lauge_niedrig:                         'Strmeldung: Fllstand zu niedrig
  let text = 3
  if hhe < 500 then goto start_blasen 'Untere Trigger-Schwelle fr
goto error_disp                        'minimale Fllhhe 


'-------------------------------------------------------------------------  

wartung:                               'Strmeldung: Wartung
  let text = 4
  let zykl_wd = 0
goto error_disp

'-------------------------------------------------------------------------  
  
error_disp:                            'Anzeige-Routine der Strmeldungen
  gosub store_zyklen
  
  error_loop:
    output rb,%00000000
    lcdwrite 1,1,"S T O E R U N G "
    on text gosub error_txt0,error_txt1,error_txt2,error_txt3,error_txt4
    set stop_led
    set hupe
    wait 300
    lcdwrite 1,1,"                "
    res stop_led
    res hupe
    wait 500
    if taste_stop = 0 then goto main   
  goto error_loop  
  
'-------------------------------------------------------------------------
  
error_txt0:                            'Strmeldungstext
  lcdwrite 2,1,"    NOT-AUS     "
return

'-------------------------------------------------------------------------

error_txt1:                            'Strmeldungstext
  lcdwrite 2,1," Tank ist leer  "
return

'-------------------------------------------------------------------------

error_txt2:                            'Strmeldungstext
  lcdwrite 2,1,"Tankdeckel fehlt"
return         

'-------------------------------------------------------------------------
 
error_txt3:                            'Strmeldungstext
  lcdwrite 2,1," Zu wenig Lauge "
return           

'-------------------------------------------------------------------------  

error_txt4:                            'Strmeldungstext
  lcdwrite 2,1,"     Wartung    "
return         

'-------------------------------------------------------------------------  

maschine_init:                         'Maschine initialisieren 
  lcdwrite 1,1,"INITIALISIERUNG "
  let sensor_flag = 0
      
  gosub watch_dog
  gosub maschine_start
      
  loop_init:
    gosub tank_check                   'Tankinhalt prfen
    gosub watch_dog
    if endschalter = 1 then goto exit_loop_init
  goto loop_init  
 
  exit_loop_init:                      'Maschine in Ausgangsposition
    gosub maschine_stop 
    if sensor_flag = 0 then goto tank_leer
    if endschalter = 1 and sensor_flag = 1 then let init_flag = 1 else let init_flag = 0 
  return

'------------------------------------------------------------------------- 
  
maschine_start:                        'Start Maschine
  gosub watch_dog
  lcdwrite 2,1,"START Motor     "
  set motor
  if endschalter = 1 then goto maschine_start    'Freifahrt vom Endschalter
  inc zyklus
  inc zykl_wd
  inc zyklen
return

'-------------------------------------------------------------------------  
  
maschine_stop:                         'Stop Maschine
  res motor
  lcdwrite 2,1,"STOP Motor      "
  wait 1000
return  

'-------------------------------------------------------------------------  
  
fan:                                   'Start / Stop Ventilator
  lcdwrite 2,1,"START Ventilator"
  set ventilator
  wait 4000
  gosub watch_dog
  lcdwrite 2,1,"STOP Ventilator "
  res ventilator
  wait 1000
  gosub watch_dog
return   
 
'-------------------------------------------------------------------------  

tank_check:                            'Tankinhalt prfen
  if sensor_flag = 1 then return
  if sensor = 0 then let sensor_flag = 1
return

'-------------------------------------------------------------------------

tank_messen:                           'Aktuellen Tankinhalt in ml messen
  start:                               'Start Messung bei Mediumkontakt
    gosub watch_dog
    if sensor = 0 then goto loop_pos_flanke1
  goto start
  
  add_hhe:                            '1900 Impulse = 80 Milli-Liter
    inc hhe                           '1900 Impulse = Tank voll
    gosub watch_dog
  return  
     
  loop_pos_flanke1:                    'Ermittlung positive Flanke Reihe 1          
    if in1 = 0 then goto loop_pos_flanke1
    if in1 = 1 then gosub add_hhe
    
  loop_pos_flanke2:                    'Ermittlung positive Flanke Reihe 2
    if in2 = 0 then goto loop_pos_flanke2
    if in2 = 1 then gosub add_hhe
    
  loop_neg_flanke1:                    'Ermittlung negative Flanke Reihe 1          
    if in1 = 1 then goto loop_neg_flanke1
    
  loop_neg_flanke2:                    'Ermittlung negative Flanke Reihe 2
    if in2 = 1 then goto loop_neg_flanke2  
    
    if taste_stop = 0 then let stop_flag = 0
    if stop_flag = 0 then set stop_led else res stop_led
    if sensor = 0 then goto loop_pos_flanke1
    
  let inhalt  = 27 * hhe               'aktuellen Tankinhalt berechnen 
  let volumen = inhalt / 633 
  if hhe < 600 then goto lauge_niedrig 'Obere Trigger-Schwelle fr 
return                                  'minimale Fllhhe 

'-------------------------------------------------------------------------  
  
start_blasen:                          'Start Seifenblasen-Produktion
  let hhe = 0
  let stop_flag = 1
  gosub maschine_start
  
  gosub tank_messen
    
  switch:
    if taste_stop = 0 then let stop_flag = 0 
    gosub watch_dog
    if stop_flag = 0 then set stop_led else res stop_led
    if endschalter = 1 then gosub maschine_stop else goto switch
    
  gosub fan
  
  gosub aktuell_info
  
  if stop_flag = 0 then gosub store_zyklen
  if stop_flag = 0 then goto loop_2
  
goto start_blasen

'-------------------------------------------------------------------------

system_info:
  lcdclear  
  lcdwrite 1,1,summen_symb,#
  lcdwrite 1,3,"Zyklen"                'Anzeige der Gesamtzyklenzahl
  lcdwrite 1,12,zyklen
  lcdwrite 2,1,summen_symb,#
  lcdwrite 2,3,"WD-Zyklen"             'Anzeige der Zyklen bis zum WD
  lcdwrite 2,12,zykl_wd 
  wait 5000
return
  
'-------------------------------------------------------------------------
  
aktuell_info:
  lcdwrite 1,1,"T"
  lcdwrite 1,2,volumen                 'Anzeige Fllstand in ml
  lcdwrite 1,5," ml     "
  lcdwrite 1,10,"n"
  lcdwrite 1,12,zyklus                 'Anzeige der aktuellen Zyklenzahl
  lcdwrite 2,1,"START STOPP INFO"  
return  


'***********************************************************************
'*****                     Hauptprogramm                           *****
'***********************************************************************

main:                                  'Hauptprogramm-Schleife
  gosub init                 
  gosub init_var
  gosub read_zyklen
  
  gosub watch_dog
  
  lcdclear
  
  loop_menu:                           'Untermen-Schleife
    gosub watch_dog
    on menu gosub loop_1, loop_2   
  goto loop_menu
 
  loop_1:                              'Untermen Initialisierung
    lcdwrite 2,1,"START"
    lcdwrite 1,1,"INITIALISIERUNG"
    if taste_start = 0 then gosub maschine_init
    gosub watch_dog
    if init_flag = 1 then let menu = 1 else let menu = 0  
  return  
  
  loop_2:                              'Untermen Seifenblasenproduktion
    gosub aktuell_info                           
    if taste_start = 0 then goto start_blasen
    if taste_info  = 0 then gosub system_info
    gosub watch_dog
  return
       
  end                                  'Ende gut, alles gut       
        
 